@@ -309,7 +309,7 @@ static PetscErrorCode  PCCompositeSetType_Composite(PC pc,PCCompositeType type)
   } else if (type ==  PC_COMPOSITE_SPECIAL) {
     pc->ops->apply          = PCApply_Composite_Special;
     pc->ops->applytranspose = NULL;
-  } else SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Unkown composite preconditioner type");
+  } else SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Unknown composite preconditioner type");
   jac->type = type;
   PetscFunctionReturn(0);
 }
@@ -410,7 +410,6 @@ static PetscErrorCode  PCCompositeGetPC_Composite(PC pc,PetscInt n,PC *subpc)
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeSetType(PC pc,PCCompositeType type)
 {
@@ -439,7 +438,6 @@ PetscErrorCode  PCCompositeSetType(PC pc,PCCompositeType type)
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeGetType(PC pc,PCCompositeType *type)
 {
@@ -463,7 +461,6 @@ PetscErrorCode  PCCompositeGetType(PC pc,PCCompositeType *type)
 
    Level: Developer
 
-.keywords: PC, set, type, composite preconditioner, additive, multiplicative
 @*/
 PetscErrorCode  PCCompositeSpecialSetAlpha(PC pc,PetscScalar alpha)
 {
@@ -487,7 +484,6 @@ PetscErrorCode  PCCompositeSpecialSetAlpha(PC pc,PetscScalar alpha)
 
    Level: Developer
 
-.keywords: PC, composite preconditioner, add
 @*/
 PetscErrorCode  PCCompositeAddPC(PC pc,PCType type)
 {
@@ -512,8 +508,6 @@ PetscErrorCode  PCCompositeAddPC(PC pc,PCType type)
 
    Level: Developer
 
-.keywords: PC, get, composite preconditioner, sub preconditioner
-
 .seealso: PCCompositeGetPC()
 @*/
 PetscErrorCode  PCCompositeGetNumberPC(PC pc,PetscInt *num)
@@ -545,8 +539,6 @@ PetscErrorCode  PCCompositeGetNumberPC(PC pc,PetscInt *num)
     To use a different operator to construct one of the inner preconditioners first call PCCompositeGetPC(), then 
             call PCSetOperators() on that PC.
 
-.keywords: PC, get, composite preconditioner, sub preconditioner
-
 .seealso: PCCompositeAddPC(), PCCompositeGetNumberPC(), PCSetOperators()
 @*/
 PetscErrorCode  PCCompositeGetPC(PC pc,PetscInt n,PC *subpc)
@@ -572,8 +564,6 @@ PetscErrorCode  PCCompositeGetPC(PC pc,PetscInt n,PC *subpc)
 
    Level: intermediate
 
-   Concepts: composing solvers
-
    Notes:
     To use a Krylov method inside the composite preconditioner, set the PCType of one or more
           inner PCs to be PCKSP.
